"use strict";
exports.__esModule = true;
var Benchmark = require('benchmark');
var child_process_1 = require("child_process");
var Vector_1 = require("../src/Vector");
var HashSet_1 = require("../src/HashSet");
var HashMap_1 = require("../src/HashMap");
var LinkedList_1 = require("../src/LinkedList");
// import immutables through require for now so that
// the immutables typescript type definitions are not
// parsed: as of typescript 2.9rc and immutables 4.0.0rc9
// they don't compile.
var imm = require('immutable');
// import * as imm from 'immutable';
var hamt = require("hamt_plus");
var hamtBase = require("hamt");
var Funkia = require("list");
var lengths = [200, 10000];
function getPrerequisites(length) {
    // https://stackoverflow.com/a/43044960/516188
    var getArray = function (length) { return Array.from({ length: length }, function () { return Math.floor(Math.random() * length); }); };
    var array = getArray(length);
    var vec = Vector_1.Vector.ofIterable(array);
    var rawhamt = hamt.empty.mutate(function (h) {
        var iterator = array[Symbol.iterator]();
        var curItem = iterator.next();
        while (!curItem.done) {
            h.set(h.size, curItem.value);
            curItem = iterator.next();
        }
    });
    var rawhamtBase = hamtBase.empty;
    var iterator = array[Symbol.iterator]();
    var curItem = iterator.next();
    while (!curItem.done) {
        rawhamtBase = rawhamtBase.set(rawhamtBase.size, curItem.value);
        curItem = iterator.next();
    }
    var list = LinkedList_1.LinkedList.ofIterable(array);
    var immList = imm.List(array);
    var funkiaList = Funkia.fromArray(array);
    var idxThreeQuarters = array.length * 3 / 4;
    var atThreeQuarters = array[idxThreeQuarters];
    var hashset = HashSet_1.HashSet.ofIterable(array);
    var immSet = imm.Set(array);
    var hashmap = HashMap_1.HashMap.ofIterable(array.map(function (x) { return [x + "", x]; }));
    var immMap = imm.Map(array.map(function (x) { return [x + "", x]; }));
    return { vec: vec, immList: immList, array: array, list: list, idxThreeQuarters: idxThreeQuarters,
        rawhamt: rawhamt, rawhamtBase: rawhamtBase, hashset: hashset, immSet: immSet, length: length, hashmap: hashmap, immMap: immMap, funkiaList: funkiaList };
}
var preReqs = lengths.map(getPrerequisites);
function _compare(preReqs, items) {
    var benchSuite = new Benchmark.Suite;
    var _loop_1 = function (item) {
        benchSuite.add(item[0], function () { return item[1](preReqs); });
    };
    for (var _i = 0, items_1 = items; _i < items_1.length; _i++) {
        var item = items_1[_i];
        _loop_1(item);
    }
    benchSuite.on('cycle', function (event) {
        console.log(String(event.target));
    }).on('complete', function () {
        console.log('Fastest is ' + this.filter('fastest').map('name') + "\n");
    }).run();
}
function compare() {
    var items = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        items[_i] = arguments[_i];
    }
    for (var i = 0; i < lengths.length; i++) {
        var length_1 = lengths[i];
        console.log("n = " + length_1);
        _compare(preReqs[i], items);
    }
}
console.log("immList, immSet are the immutablejs list,set... https://facebook.github.io/immutable-js/");
console.log("funkiaList is the list from https://github.com/funkia/list");
process.stdout.write("node version: ");
child_process_1.execSync("node --version", { stdio: [0, 1, 2] });
console.log();
compare(['Vector.toArray', function (p) { return p.vec.toArray(); }], ['LinkedList.toArray', function (p) { return p.list.toArray(); }], ['immList.toArray', function (p) { return p.immList.toArray(); }], ['funkiaList.toArray', function (p) { return Funkia.toArray(p.funkiaList); }]);
compare(['Vector.take', function (p) { return p.vec.take(p.idxThreeQuarters); }], ['Array.slice', function (p) { return p.array.slice(0, p.idxThreeQuarters); }], ['immList.take', function (p) { return p.immList.take(p.idxThreeQuarters); }], ['LinkedList.take', function (p) { return p.list.take(p.idxThreeQuarters); }], ['funkiaList.take', function (p) { return Funkia.take(p.idxThreeQuarters, p.funkiaList); }]);
compare(['Vector.drop', function (p) { return p.vec.drop(p.idxThreeQuarters); }], ['Array.slice', function (p) { return p.array.slice(p.idxThreeQuarters); }], ['immList.slice', function (p) { return p.immList.slice(p.idxThreeQuarters); }], ['LinkedList.drop', function (p) { return p.list.drop(p.idxThreeQuarters); }], ['funkiaList.drop', function (p) { return Funkia.drop(p.idxThreeQuarters, p.funkiaList); }]);
compare(['Vector.filter', function (p) { return p.vec.filter(function (x) { return x % 2 === 0; }); }], ['Array.filter', function (p) { return p.array.filter(function (x) { return x % 2 === 0; }); }], ['immList.filter', function (p) { return p.immList.filter(function (x) { return x % 2 === 0; }); }], ['LinkedList.filter', function (p) { return p.list.filter(function (x) { return x % 2 === 0; }); }], ['funkiaList.filter', function (p) { return Funkia.filter(function (x) { return x % 2 === 0; }, p.funkiaList); }]);
compare(['Vector.map', function (p) { return p.vec.map(function (x) { return x * 2; }); }], ['Array.map', function (p) { return p.array.map(function (x) { return x * 2; }); }], ['immList.map', function (p) { return p.immList.map(function (x) { return x * 2; }); }], ['LinkedList.map', function (p) { return p.list.map(function (x) { return x * 2; }); }], ['funkiaList.map', function (p) { return Funkia.map(function (x) { return x * 2; }, p.funkiaList); }]);
compare(['Vector.find', function (p) { return p.vec.find(function (x) { return x === p.idxThreeQuarters; }); }], ['Array.find', function (p) { return p.array.find(function (x) { return x === p.idxThreeQuarters; }); }], ['immList.find', function (p) { return p.immList.find(function (x) { return x === p.idxThreeQuarters; }); }], ['LinkedList.find', function (p) { return p.list.find(function (x) { return x === p.idxThreeQuarters; }); }], ['funkiaList.find', function (p) { return Funkia.find(function (x) { return x === p.idxThreeQuarters; }, p.funkiaList); }]);
compare(['Vector.ofIterable', function (p) { return Vector_1.Vector.ofIterable(p.array); }], ['rawhamt.build from iterable', function (p) {
        hamt.empty.mutate(function (h) {
            var iterator = p.array[Symbol.iterator]();
            var curItem = iterator.next();
            while (!curItem.done) {
                h.set(h.size, curItem.value);
                curItem = iterator.next();
            }
        });
    }], ['rawhamt.build from array', function (p) {
        hamt.empty.mutate(function (h) {
            for (var i = 0; i < p.array.length; i++) {
                h.set(i, p.array[i]);
            }
        });
    }], ['rawhamtBase.build from iterable', function (p) {
        var rawhamtBase = hamtBase.empty;
        var iterator = p.array[Symbol.iterator]();
        var curItem = iterator.next();
        while (!curItem.done) {
            rawhamtBase = rawhamtBase.set(rawhamtBase.size, curItem.value);
            curItem = iterator.next();
        }
    }], ['LinkedList.ofIterable', function (p) { return LinkedList_1.LinkedList.ofIterable(p.array); }], ['immList.ofIterable', function (p) { return imm.List(p.array); }], ['funkiaList.ofArray', function (p) { return Funkia.fromArray(p.array); }]);
compare(['Vector.get(i)', function (p) { return p.vec.get(p.length / 2); }], ['rawhamt.get(i)', function (p) { return p.rawhamt.get(p.length / 2); }], ['rawhamtBase.get(i)', function (p) { return p.rawhamtBase.get(p.length / 2); }], ['LinkedList.get(i)', function (p) { return p.list.get(p.length / 2); }], ['Array.get(i)', function (p) { return p.array[p.length / 2]; }], ['immList.get(i)', function (p) { return p.immList.get(p.length / 2); }], ['funkiaList.get(i)', function (p) { return Funkia.nth(p.length / 2, p.funkiaList); }]);
compare(['Vector.flatMap', function (p) { return p.vec.flatMap(function (x) { return Vector_1.Vector.of(1, 2); }); }], ['LinkedList.flatMap', function (p) { return p.list.flatMap(function (x) { return LinkedList_1.LinkedList.of(1, 2); }); }], ['immList.flatMap', function (p) { return p.immList.flatMap(function (x) { return imm.List([1, 2]); }); }], ['funkiaList.chain', function (p) { return Funkia.chain(function (x) { return Funkia.list(1, 2); }, p.funkiaList); }]);
compare(['Vector.reverse', function (p) { return p.vec.reverse(); }], ['Array.reverse', function (p) { return p.array.reverse(); }], ['immList.reverse', function (p) { return p.immList.reverse(); }], ['LinkedList.reverse', function (p) { return p.list.reverse(); }], ['funkiaList.reverse', function (p) { return Funkia.reverse(p.funkiaList); }]);
compare(['Vector.groupBy', function (p) { return p.vec.groupBy(function (x) { return x % 2; }); }], ['LinkedList.groupBy', function (p) { return p.list.groupBy(function (x) { return x % 2; }); }], ['immList.groupBy', function (p) { return p.immList.groupBy(function (x) { return x % 2; }); }]);
compare(['Vector.append', function (p) {
        var v = Vector_1.Vector.empty();
        for (var _i = 0, _a = p.array; _i < _a.length; _i++) {
            var item = _a[_i];
            v = v.append(item);
        }
    }], ['Array.push', function (p) {
        var v = [];
        for (var _i = 0, _a = p.array; _i < _a.length; _i++) {
            var item = _a[_i];
            v.push(item);
        }
    }], ['immList.push', function (p) {
        // let v = imm.List<number>();
        var v = imm.List();
        for (var _i = 0, _a = p.array; _i < _a.length; _i++) {
            var item = _a[_i];
            v = v.push(item);
        }
    }], ['LinkedList.append', function (p) {
        var v = LinkedList_1.LinkedList.empty();
        for (var _i = 0, _a = p.array; _i < _a.length; _i++) {
            var item = _a[_i];
            v = v.append(item);
        }
    }], ['Funkia.append', function (p) {
        var v = Funkia.empty();
        for (var _i = 0, _a = p.array; _i < _a.length; _i++) {
            var item = _a[_i];
            v = Funkia.append(item, v);
        }
    }]);
compare(['Vector.prepend', function (p) {
        var v = Vector_1.Vector.empty();
        for (var _i = 0, _a = p.array; _i < _a.length; _i++) {
            var item = _a[_i];
            v = v.prepend(item);
        }
    }], ['Array.unshift', function (p) {
        var v = [];
        for (var _i = 0, _a = p.array; _i < _a.length; _i++) {
            var item = _a[_i];
            v.unshift(item);
        }
    }], ['immList.unshift', function (p) {
        // let v = imm.List<number>();
        var v = imm.List();
        for (var _i = 0, _a = p.array; _i < _a.length; _i++) {
            var item = _a[_i];
            v = v.unshift(item);
        }
    }], ['LinkedList.prepend', function (p) {
        var v = LinkedList_1.LinkedList.empty();
        for (var _i = 0, _a = p.array; _i < _a.length; _i++) {
            var item = _a[_i];
            v = v.prepend(item);
        }
    }], ['Funkia.prepend', function (p) {
        var v = Funkia.empty();
        for (var _i = 0, _a = p.array; _i < _a.length; _i++) {
            var item = _a[_i];
            v = Funkia.prepend(item, v);
        }
    }]);
function iterateOn(coll) {
    var it = coll[Symbol.iterator]();
    var item = it.next();
    while (!item.done) {
        item = it.next();
    }
}
compare(['Vector.iterate', function (p) { return iterateOn(p.vec); }], ['Array.iterate', function (p) { return iterateOn(p.array); }], ['immList.iterate', function (p) { return iterateOn(p.immList); }], ['LinkedList.iterate', function (p) { return iterateOn(p.list); }], ['Funkia.iterate', function (p) { return iterateOn(p.funkiaList); }]);
compare(['Vector.appendAll', function (p) { return p.vec.appendAll(p.vec); }], ['Array.appendAll', function (p) { return p.array.concat(p.array); }], ['immList.appendAll', function (p) { return p.immList.concat(p.immList); }], ['LinkedList.appendAll', function (p) { return p.list.appendAll(p.list); }], ['Funkia.concat', function (p) { return Funkia.concat(p.funkiaList, p.funkiaList); }]);
compare(['Vector.prependAll', function (p) { return p.vec.prependAll(p.vec); }], ['Array.prependAll', function (p) { return p.array.concat(p.array); }], ['LinkedList.prependAll', function (p) { return p.list.prependAll(p.list); }]);
compare(['Vector.foldLeft', function (p) { return p.vec.foldLeft(0, function (acc, i) { return acc + i; }); }], ['Array.foldLeft', function (p) { return p.array.reduce(function (acc, i) { return acc + i; }); }], ['immList.foldLeft', function (p) { return p.immList.reduce(function (acc, i) { return acc + i; }, 0); }], ['LinkedList.foldLeft', function (p) { return p.vec.foldLeft(0, function (acc, i) { return acc + i; }); }], ['Funkia.foldl', function (p) { return Funkia.foldl(function (i, acc) { return acc + i; }, 0, p.funkiaList); }]);
compare(['Vector.foldRight', function (p) { return p.vec.foldRight(0, function (i, acc) { return acc + i; }); }], ['immList.foldRight', function (p) { return p.immList.reduceRight(function (acc, i) { return acc + i; }, 0); }], ['LinkedList.foldRight', function (p) { return p.vec.foldRight(0, function (i, acc) { return acc + i; }); }], ['Funkia.foldr', function (p) { return Funkia.foldr(function (i, acc) { return acc + i; }, 0, p.funkiaList); }]);
compare(['HashSet.ofIterable', function (p) { return HashSet_1.HashSet.ofIterable(p.array); }], ['immSet', function (p) { return imm.Set(p.array); }]);
compare(['HashSet.ofIterable (from vector)', function (p) { return HashSet_1.HashSet.ofIterable(p.vec); }], ['immSet (from vector)', function (p) { return imm.Set(p.vec); }]);
compare(['HashSet.contains', function (p) { return p.hashset.contains(p.array[Math.floor(Math.random() * p.array.length)]); }], ['immSet.contains', function (p) { return p.immSet.contains(p.array[Math.floor(Math.random() * p.array.length)]); }]);
compare(['HashSet.filter', function (p) { return p.hashset.filter(function (x) { return x < p.length / 2; }); }], ['immSet.filter', function (p) { return p.immSet.filter(function (x) { return x < p.length / 2; }); }]);
compare(['HashMap.ofIterable', function (p) {
        return HashMap_1.HashMap.ofIterable(p.array.map(function (x) { return [x + "", x]; }));
    }], ['immMap', function (p) { return imm.Map(p.array.map(function (x) { return [x + "", x]; })); }]);
compare(['HashMap.ofIterable (from vector)', function (p) {
        return HashMap_1.HashMap.ofIterable(p.vec.map(function (x) { return [x + "", x]; }));
    }], ['immMap (from vector)', function (p) {
        return imm.Map(p.vec.map(function (x) { return [x + "", x]; }));
    }]);
compare(['HashMap.get', function (p) { return p.hashmap.get(p.array[Math.floor(Math.random() * p.array.length)] + ""); }], ['immMap.get', function (p) { return p.immMap.get(p.array[Math.floor(Math.random() * p.array.length)] + ""); }]);
compare(['HashMap.filter', function (p) { return p.hashmap.filter(function (k, v) { return parseInt(k) < p.length / 2; }); }], ['immMap.filter', function (p) { return p.immMap.filter(function (v, k) { return parseInt(k) < p.length / 2; }); }]);
//# sourceMappingURL=bench.js.map