"use strict";
exports.__esModule = true;
var fs_1 = require("fs");
var Vector_1 = require("../../src/Vector");
var Function_1 = require("../../src/Function");
var Predicate_1 = require("../../src/Predicate");
var helpers = require("./helpers");
var classesFolder = "apidoc/classes/";
/**
 * we work on lines-level in the text output, not xml tags
 * sometimes we have in one parentne "<parent>..</parent>"
 * but we also want to support "<parent>\n...\n</parent>"
 */
function putStaticMethodsOnTop(parentTag, parentMarker, childTag, linesByIndent) {
    var _a = helpers.linesByIndentGetTagContents(linesByIndent, parentTag, function (l) { return l.indexOf(parentMarker) >= 0; }), beforeIndexContent = _a[0], indexContent = _a[1], afterIndexContent = _a[2];
    var openTag = Predicate_1.Predicate.of(function (l) { return l.contents.indexOf("<" + childTag) >= 0; });
    // the marker parent might be of the same tag, so keep it always.
    var beforeFirstTag = indexContent.take(1)
        .appendAll(indexContent.drop(1).takeWhile(openTag.negate()));
    // the marker parent might be of the same tag, so keep it always.
    var afterLastTag = indexContent.reverse().take(1)
        .appendAll(indexContent
        .reverse().drop(1)
        .takeWhile(function (l) { return l.contents.indexOf("</" + childTag) < 0; }))
        .reverse();
    var staticOrNot = indexContent
        .drop(beforeFirstTag.length())
        .dropRight(afterLastTag.length())
        // group the li tags and their contents together, even if they span
        // across several lines like <parent>\n..\n</parent>
        .foldLeft(Vector_1.Vector.empty(), function (sofar, cur) { return openTag(cur) ?
        // open li tag => start a new entry
        sofar.append(Vector_1.Vector.of(cur)) :
        // not open li tag => continue the previous entry
        sofar.dropRight(1).append(sofar.last().getOrThrow().append(cur)); })
        .partition(function (l) { return l.head().getOrThrow().contents.indexOf("tsd-is-static") >= 0; });
    return beforeIndexContent
        .appendAll(beforeFirstTag)
        .appendAll(staticOrNot[0].flatMap(function (x) { return x; }))
        .appendAll(staticOrNot[1].flatMap(function (x) { return x; }))
        .appendAll(afterLastTag)
        .appendAll(afterIndexContent);
}
function classPutStaticMethodsOnTop(classfilename) {
    var lines = helpers.fileGetLinesByIndent(classesFolder + classfilename);
    var fnStatic = Function_1.Function4.of(putStaticMethodsOnTop);
    var transform = fnStatic.apply3("ul", "tsd-index-list", "li")
        .andThen(fnStatic.apply3("li", "current tsd-kind-class", "li"))
        .andThen(fnStatic.apply3("section", "tsd-panel-group tsd-member-group", "section"));
    fs_1.writeFileSync(classesFolder + classfilename, helpers.linesByIndentStr(transform(lines)));
}
function putClassStaticMethodsOnTop() {
    var classFiles = Vector_1.Vector.ofIterable(fs_1.readdirSync(classesFolder));
    classFiles.forEach(classPutStaticMethodsOnTop);
}
exports.putClassStaticMethodsOnTop = putClassStaticMethodsOnTop;
//# sourceMappingURL=classes.js.map