"use strict";
exports.__esModule = true;
var Vector_1 = require("../../src/Vector");
var HashSet_1 = require("../../src/HashSet");
var fs_1 = require("fs");
var helpers = require("./helpers");
// group classes & interfaces by category.
var CATEGORIES = Vector_1.Vector.of(["Control", Vector_1.Vector.of("Either", "Option", "Lazy", "Function", "Predicate", "Future")], ["Collection", Vector_1.Vector.of("Collection", "Foldable", "IMap", "HashMap", "ISet", "HashSet", "Seq", "LinkedList", "Stream", "Vector", "Tuple2", "SortOnSpec", "SortBySpec")], ["Core", Vector_1.Vector.of("Comparison", "Value", "Contract")]);
function getSectionHeader(sectionName) {
    return helpers.indent(6) + "<section class=\"tsd-index-section\">" +
        (helpers.indent(7) + "<h3>" + sectionName + "</h3>") +
        (helpers.indent(7) + "<ul class=\"tsd-index-list\">\n");
}
function getSectionFooter() {
    return helpers.indent(7) + "</ul>" +
        (helpers.indent(6) + "</section>");
}
// improve the typedoc 'globals' screen by
// grouping the classes, interfaces by "categories"
// and not by type (class, interface,...) as typedoc
// does out of the box.
// i should rather try to improve typedoc but..
//
// I rather not using typescript external modules
// because there are several options and I think
// that may assume too much about how users of the
// library want to consume it.
// So everything is in the base namespace, but I can
// at least do some grouping in the apidocs.
function groupGlobalsByCategory() {
    // we'll modify the 'globals' typedoc file.
    var lines = helpers.fileGetLinesByIndent("apidoc/globals.html");
    // i'm interested in the part within 'tsd-index-content'
    var _a = helpers.linesByIndentGetTagContents(lines, "div", function (l) { return l.indexOf("tsd-index-content") >= 0; }), beforeIndexContent = _a[0], indexContent = _a[1], afterIndexContent = _a[2];
    // right now i have the part of interest. I can regenerate the wrapping items
    // the items which interest me are the leaves, which are <li> tags,
    // and I'll group them by their name (class name, interface name...).
    var liRows = indexContent
        .map(function (l) { return l.contents; })
        .filter(function (t) { return t.indexOf("<li") >= 0; })
        .arrangeBy(function (row) { return helpers.requireNotNull(row.match(/>([\w<>]+)<\//))[1].replace(/<wbr>/g, ""); })
        .getOrThrow("globals.arrangeBy failed!");
    // start preparing the new contents for the indexContent
    var newIndexContent = helpers.indent(5) + "<div class=\"tsd-index-content\">";
    var allKnownElements = CATEGORIES
        .map(function (c) { return c[1]; })
        .flatMap(function (x) { return x; })
        .transform(HashSet_1.HashSet.ofIterable);
    var missingElements = liRows.keySet().diff(allKnownElements);
    if (!missingElements.isEmpty()) {
        throw "Missing the following elements: " + missingElements;
    }
    CATEGORIES.forEach(function (_a) {
        var name = _a[0], elements = _a[1];
        newIndexContent += getSectionHeader(name);
        var rows = elements.map(function (elt) { return liRows.get(elt).getOrThrow("can't find row for " + elt); });
        newIndexContent += rows.mkString("\n");
        newIndexContent += getSectionFooter();
    });
    // conclude the new contents for the indexContent
    newIndexContent += helpers.indent(5) + "</div>\n"; // close tsd-index-content
    // overwrite globals.html -- first the text before the indexContent,
    // then the modified indexContent, then the rest.
    fs_1.writeFileSync('apidoc/globals.html', helpers.linesByIndentStr(beforeIndexContent) +
        newIndexContent +
        helpers.linesByIndentStr(afterIndexContent));
}
exports.groupGlobalsByCategory = groupGlobalsByCategory;
//# sourceMappingURL=globals.js.map