"use strict";
exports.__esModule = true;
var Stream_1 = require("../../src/Stream");
var Vector_1 = require("../../src/Vector");
var fs_1 = require("fs");
function requireNotNull(x) {
    if (x === null) {
        throw "requireNotNull got null!";
    }
    return x;
}
exports.requireNotNull = requireNotNull;
function indent(count) {
    return "\n" + Stream_1.Stream.continually(function () { return "\t"; })
        .take(count).mkString("");
}
exports.indent = indent;
function fileGetLinesByIndent(fname) {
    var contents = fs_1.readFileSync(fname).toString();
    return Vector_1.Vector.ofIterable(contents.split("\n"))
        .map(function (l) { return ({ indent: l.length - l.trim().length, contents: l }); });
}
exports.fileGetLinesByIndent = fileGetLinesByIndent;
/**
 * extract a tag from the linesbyindent. You tell me the tag name,
 * how to match the start tag, I return to you the rows before, the rows within that
 * tag (depth>=tagDepth) and the rows after.
 * returns [before, atTag, after]
 */
function linesByIndentGetTagContents(lines, tagName, startPredicate) {
    // split the parts before & after using string the predicate & indentation.
    var _a = lines.span(function (l) { return !startPredicate(l.contents); }), before = _a[0], atTag = _a[1];
    var indentAtTag = atTag.head().getOrThrow().indent;
    var _b = atTag.span(function (l) { return (l.indent > indentAtTag) ||
        (l.indent <= indentAtTag && l.contents.indexOf("</" + tagName) < 0); }), tagContents = _b[0], after = _b[1];
    if (!after.isEmpty()) {
        // i want the closing tag also in 'atTag' but I rejected it in the span,
        // so take it from after
        return [before, tagContents.append(after.head().getOrThrow()), after.drop(1)];
    }
    return [before, tagContents, after];
}
exports.linesByIndentGetTagContents = linesByIndentGetTagContents;
function linesByIndentStr(linesByIndent) {
    return linesByIndent.map(function (l) { return l.contents; }).mkString("\n");
}
exports.linesByIndentStr = linesByIndentStr;
//# sourceMappingURL=helpers.js.map