"use strict";
exports.__esModule = true;
var HashMap_1 = require("../src/HashMap");
var Option_1 = require("../src/Option");
var assert = require("assert");
/**
 * @hidden
 */
function runTests(seqName, of, empty) {
    describe(seqName + " manipulation", function () {
        it("arrangeBy works, positive case", function () { return assert.ok(HashMap_1.HashMap.of(["a", "aadvark"], ["b", "baseball"]).equals(of("aadvark", "baseball").arrangeBy(function (x) { return x[0]; }).getOrThrow())); });
        it("arrangeBy works, negative case", function () { return assert.ok(of("aadvark", "aaseball").arrangeBy(function (x) { return x[0]; }).isNone()); });
        it("arrangeBy works, empty seq", function () { return assert.ok(HashMap_1.HashMap.empty().equals(empty().arrangeBy(function (x) { return x[0]; }).getOrThrow())); });
        it("groupBy works", function () { return assert.ok(HashMap_1.HashMap.empty().put(0, of(2, 4)).put(1, of(1, 3))
            .equals(of(1, 2, 3, 4).groupBy(function (x) { return x % 2; }))); });
        it("reduce works", function () { return assert.equal(6, of(1, 2, 3).reduce(function (a, b) { return a + b; }).getOrThrow()); });
        it("reduce works on an empty collection", function () { return assert.ok(Option_1.Option.none().equals(empty().reduce(function (a, b) { return a + b; }))); });
        it("minOn works", function () { return assert.equal(2, of(2, 3, 4).minOn(function (x) { return x; }).getOrThrow()); });
        it("minOn works on the empty collection", function () { return assert.ok(empty().minOn(function (x) { return x; }).isNone()); });
        it("minBy works", function () { return assert.equal(4, of(2, 3, 4).minBy(function (x, y) { return x - y; }).getOrThrow()); });
        it("minBy works on the empty collection", function () { return assert.ok(empty().minBy(function (x, y) { return x - y; }).isNone()); });
        it("maxOn works", function () { return assert.equal(4, of(2, 3, 4).maxOn(function (x) { return x; }).getOrThrow()); });
        it("maxOn works on the empty collection", function () { return assert.ok(empty().maxOn(function (x) { return x; }).isNone()); });
        it("maxBy works", function () { return assert.equal(2, of(2, 3, 4).maxBy(function (x, y) { return x - y; }).getOrThrow()); });
        it("maxBy works on the empty collection", function () { return assert.ok(empty().maxBy(function (x, y) { return x - y; }).isNone()); });
        it("sumOn works", function () { return assert.equal(6, of(1, 2, 3).sumOn(function (x) { return x; })); });
        it("sumOn works on the empty collection", function () { return assert.equal(0, empty().sumOn(function (x) { return x; })); });
    });
}
exports.runTests = runTests;
//# sourceMappingURL=Collection.js.map