"use strict";
exports.__esModule = true;
var HashMap_1 = require("../src/HashMap");
var HashSet_1 = require("../src/HashSet");
var Vector_1 = require("../src/Vector");
var LinkedList_1 = require("../src/LinkedList");
var Option_1 = require("../src/Option");
var Stream_1 = require("../src/Stream");
var Tuple2_1 = require("../src/Tuple2");
var Comparison_1 = require("../src/Comparison");
var SampleData_1 = require("./SampleData");
var TestHelpers_1 = require("./TestHelpers");
var assert = require("assert");
describe("hashmap construction basic sanity tests", function () {
    it("should overwrite values with the same key", function () { return assert.ok(HashMap_1.HashMap.empty().put(5, "test").put(5, "test1")
        .equals(HashMap_1.HashMap.empty().put(5, "test1"))); });
    it("should overwrite values with the same key with custom types", function () { return assert.ok(HashMap_1.HashMap.empty()
        .put(new SampleData_1.MyClass("a", 1), "test")
        .put(new SampleData_1.MyClass("a", 1), "test1")
        .put(new SampleData_1.MyClass("a", 2), "test1").equals(HashMap_1.HashMap.empty()
        .put(new SampleData_1.MyClass("a", 1), "test1")
        .put(new SampleData_1.MyClass("a", 2), "test1"))); });
    it("should overwrite values with the same key with custom types when created with of()", function () { return assert.ok(HashMap_1.HashMap.empty()
        .put(new SampleData_1.MyClass("a", 1), "test")
        .put(new SampleData_1.MyClass("a", 1), "test1")
        .put(new SampleData_1.MyClass("a", 2), "test1").equals(HashMap_1.HashMap.of([new SampleData_1.MyClass("a", 1), "test1"])
        .put(new SampleData_1.MyClass("a", 2), "test1"))); });
    it("should overwrite values with the same key with custom types when created with ofIterable()", function () { return assert.ok(HashMap_1.HashMap.ofIterable([
        [new SampleData_1.MyClass("a", 1), "test"],
        [new SampleData_1.MyClass("a", 2), "test1"]
    ])
        .put(new SampleData_1.MyClass("a", 1), "test1")
        .equals(HashMap_1.HashMap.of([new SampleData_1.MyClass("a", 1), "test1"])
        .put(new SampleData_1.MyClass("a", 2), "test1"))); });
    it("should support map as a key itself", function () { return assert.ok(HashMap_1.HashMap.empty()
        .put(HashMap_1.HashMap.empty().put("hello", 1), 6)
        .put(HashMap_1.HashMap.empty().put("hello", 1), 7)
        .put(HashMap_1.HashMap.empty().put("bye", 1), 7)
        .equals(HashMap_1.HashMap.empty()
        .put(HashMap_1.HashMap.empty().put("hello", 1), 7)
        .put(HashMap_1.HashMap.empty().put("bye", 1), 7))); });
    it("should build with of", function () { return assert.ok(HashMap_1.HashMap.empty().put(1, "a").put(2, "b").equals(HashMap_1.HashMap.of([1, "a"], [2, "b"]))); });
    it("should build with ofIterable", function () { return assert.ok(HashMap_1.HashMap.empty().put(1, "a").put(2, "b").equals(HashMap_1.HashMap.ofIterable(Vector_1.Vector.of([1, "a"], [2, "b"])))); });
    it("should put with merge", function () { return assert.ok(HashMap_1.HashMap.empty()
        .put(5, "test").putWithMerge(5, "a", function (a, b) { return a + b; })
        .equals(HashMap_1.HashMap.empty().put(5, "testa"))); });
    it("should enforce the callback parameter order on put with merge", function () { return assert.ok(HashMap_1.HashMap.of([1, "2"], [2, "4"])
        .putWithMerge(2, "a", function (a, b) { return a; })
        .equals(HashMap_1.HashMap.of([1, "2"], [2, "4"]))); });
    it("should mergeWith", function () { return assert.ok(HashMap_1.HashMap.empty().put(1, "a").put(2, "bc").put(3, "d")
        .equals(HashMap_1.HashMap.empty().put(1, "a").put(2, "b")
        .mergeWith(HashMap_1.HashMap.empty().put(2, "c").put(3, "d"), function (v1, v2) { return v1 + v2; }))); });
    it("should enforce the callback parameter order on mergeWith", function () { return assert.ok(HashMap_1.HashMap.of([1, 2], [2, 4], [3, 5]).equals(HashMap_1.HashMap.of([1, 2], [2, 4]).mergeWith(HashMap_1.HashMap.of([1, 3], [2, 2], [3, 5]), function (x, y) { return x; }))); });
    it("should remove keys", function () { return assert.ok(HashMap_1.HashMap.of([2, "b"]).equals(HashMap_1.HashMap.of([1, "a"], [2, "b"]).remove(1))); });
    it("should allow to remove an inexisting key", function () { return assert.ok(HashMap_1.HashMap.of([1, "a"], [2, "b"]).equals(HashMap_1.HashMap.of([1, "a"], [2, "b"]).remove(3))); });
    it("should allow to remove from the empty map", function () { return assert.ok(HashMap_1.HashMap.empty().equals(HashMap_1.HashMap.of().remove(1))); });
    it("should build from a string object dictionary", function () { return assert.ok(HashMap_1.HashMap.of(["a", 1], ["b", 2]).equals(HashMap_1.HashMap.ofObjectDictionary({ a: 1, b: 2 }))); });
    it("should build from an int object dictionary", function () { return assert.ok(HashMap_1.HashMap.of(["1", "a"], ["2", "b"]).equals(HashMap_1.HashMap.ofObjectDictionary({ 1: "a", 2: "b" }))); });
    it("should strip dictionary values with undefined", function () { return assert.ok(HashMap_1.HashMap.of(["1", "a"]).equals(HashMap_1.HashMap.ofObjectDictionary({ 1: "a", 2: undefined }))); });
    it("should build from an enum object dictionary", function () {
        var s = { "a": 1, "b": 2, "c": 3 };
        var got = HashMap_1.HashMap.ofObjectDictionary(s);
        assert.ok(HashMap_1.HashMap.of(["a", 1], ["b", 2], ["c", 3]).equals(got));
    });
    it("should build from a partial enum object dictionary", function () {
        var s = { "a": 1 };
        var got = HashMap_1.HashMap.ofObjectDictionary(s);
        assert.ok(HashMap_1.HashMap.of(["a", 1]).equals(got));
    });
    it("should fail at runtime on key without equality", function () { return assert.throws(function () {
        return HashMap_1.HashMap.empty().put({ field1: 'test', field2: -1 }, "value1");
    }); });
});
describe("hashmap equality", function () {
    it("empty should be equal with empty", function () {
        return assert.ok(HashMap_1.HashMap.empty().equals(HashMap_1.HashMap.empty()));
    });
    it("non empty should be not be equal with empty", function () {
        return assert.ok(!HashMap_1.HashMap.empty().put(1, "t").equals(HashMap_1.HashMap.empty()));
    });
    it("empty should be not be equal with non empty", function () {
        return assert.ok(!HashMap_1.HashMap.empty().equals(HashMap_1.HashMap.empty().put(1, "t")));
    });
    it("doesn't throw when given another type on equals", function () { return assert.equal(false, HashMap_1.HashMap.empty().put(1, 2).equals([1, 2])); });
    it("doesn't throw when given null on equals", function () { return assert.equal(false, HashMap_1.HashMap.empty().put(1, 2).equals(null)); });
    it("empty doesn't throw when given another type on equals", function () { return assert.equal(false, HashMap_1.HashMap.empty().equals([1, 2])); });
    it("empty doesn't throw when given null on equals", function () { return assert.equal(false, HashMap_1.HashMap.empty().equals(null)); });
    it("should refuse keys which don't offer true equality (neither equals nor hashcode)", function () { return assert.throws(function () { return HashMap_1.HashMap.of([{ name: "" }, 1]); }); });
    it("should refuse keys which don't offer true equality (no hashcode)", function () { return assert.throws(function () { return HashMap_1.HashMap.of([{ name: "", equals: function (x, y) { return true; } }, 1]); }); });
    it("should refuse keys which don't offer true equality (no equals)", function () { return assert.throws(function () { return HashMap_1.HashMap.of([{ name: "", hashCode: function () { return 1; } }, 1]); }); });
    it("should refuse keys which don't offer true equality", function () { return assert.throws(function () { return HashMap_1.HashMap.of([Vector_1.Vector.of([1]), "value"]); }); });
    it("should refuse keys which don't offer true equality (vector)", function () { return assert.throws(function () { return HashMap_1.HashMap.empty().put(Vector_1.Vector.of(Vector_1.Vector.of([1])), "value"); }); });
    it("should refuse keys which don't offer true equality (stream)", function () { return assert.throws(function () { return HashMap_1.HashMap.of([Stream_1.Stream.of([1]), "value"]); }); });
    it("should refuse keys which don't offer true equality (option)", function () { return assert.throws(function () { return HashMap_1.HashMap.of([Option_1.Option.of([1]), "value"]); }); });
    it("should refuse keys which don't offer true equality (tuple2)", function () { return assert.throws(function () { return HashMap_1.HashMap.of([Tuple2_1.Tuple2.of(1, Vector_1.Vector.of([1])), "value"]); }); });
    it("should refuse keys which don't offer true equality (hashmap)", function () { return assert.throws(function () { return HashMap_1.HashMap.of([HashMap_1.HashMap.of(["a", [1]]), "value"]); }); });
    it("should fail compilation on an obviously bad key type", function () {
        return TestHelpers_1.assertFailCompile("HashMap.of([[1], 'test'])", "Argument of type \'[number[], string]\' is not assignable to parameter");
    });
});
describe("hashmap - toString should be nicely formatted", function () {
    it("should format strings and numbers", function () { return assert.equal("HashMap(key1: 6, key2: 7, key3: null)", "" + HashMap_1.HashMap.empty().put("key1", 6).put("key2", 7).put("key3", null)); });
    it("should format custom classes", function () { return assert.equal("HashMap(key1: {field1: test, field2: -1})", "" + HashMap_1.HashMap.empty().put("key1", new SampleData_1.MyClass('test', -1))); });
    it("should format cust class keys as well", function () { return assert.equal("HashMap({\"field1\":\"test\",\"field2\":-1}: 'value1')", "" + HashMap_1.HashMap.empty().put({
        field1: 'test',
        field2: -1,
        equals: function (b) { return this.field1 === b.field1 && this.field2 === b.field2; },
        hashCode: function () { return Comparison_1.fieldsHashCode(this.field1, this.field2); }
    }, "value1")); });
});
describe("hashmap extract values", function () {
    it("should retrieve values", function () { return assert.ok(HashMap_1.HashMap.empty().put("key1", 6).get("key1").contains(6)); });
    it("should not find missing values", function () { return assert.ok(HashMap_1.HashMap.empty().put("key1", 6).get("key2").isNone()); });
    it("should retrieve nulls", function () { return assert.ok(HashMap_1.HashMap.empty().put("key1", null).get("key1").contains(null)); });
    it("should get empty keySet", function () { return assert.ok(HashSet_1.HashSet.empty().equals(HashMap_1.HashMap.empty().keySet())); });
    it("should get non-empty keySet", function () { return assert.ok(HashSet_1.HashSet.of("a", "c").equals(HashMap_1.HashMap.empty().put("a", "b").put("c", "d").keySet())); });
    it("should get empty valueIterable", function () { return assert.ok(HashSet_1.HashSet.empty().equals(HashSet_1.HashSet.ofIterable(HashMap_1.HashMap.empty().valueIterable()))); });
    it("should get non-empty valueIterable", function () { return assert.ok(HashSet_1.HashSet.of("b", "d").equals(HashSet_1.HashSet.ofIterable(HashMap_1.HashMap.empty().put("a", "b").put("c", "d").valueIterable()))); });
    it("supports iterator", function () {
        var total = 0;
        var letters = [];
        var iterator = HashMap_1.HashMap.empty()
            .put("a", 1).put("b", 6).put("c", 3)[Symbol.iterator]();
        var curItem = iterator.next();
        while (!curItem.done) {
            letters.push(curItem.value[0]);
            total += curItem.value[1];
            curItem = iterator.next();
        }
        assert.equal(10, total);
        letters.sort();
        assert.deepEqual(["a", "b", "c"], letters);
    });
    it("supports empty iterator", function () {
        var total = 0;
        var letters = [];
        var iterator = HashMap_1.HashMap.empty()[Symbol.iterator]();
        var curItem = iterator.next();
        while (!curItem.done) {
            letters.push(curItem.value[0]);
            total += curItem.value[1];
            curItem = iterator.next();
        }
        assert.equal(0, total);
        assert.deepEqual([], letters);
    });
    it("correct returns single positive case", function () { return assert.deepEqual([5, "a"], HashMap_1.HashMap.of([5, "a"]).single().getOrThrow()); });
    it("correct returns single negative case", function () { return assert.ok(HashMap_1.HashMap.of([5, "a"], [6, "b"]).single().isNone()); });
    it("correct returns single empty map", function () { return assert.ok(HashMap_1.HashMap.empty().single().isNone()); });
});
describe("hashmap transformation", function () {
    it("should transform through map", function () { return assert.ok(HashMap_1.HashMap.empty().put(12, "key1").put(6, "key2").equals(HashMap_1.HashMap.empty().put("key1", 6).put("key2", 3).map(function (k, v) { return [v * 2, k]; }))); });
    it("should transform through empty map", function () { return assert.ok(HashMap_1.HashMap.empty().equals(HashMap_1.HashMap.empty().map(function (k, v) { return [v * 2, k]; }))); });
    it("flatMap works", function () { return assert.deepEqual([["a", 1], ["b", 2], ["c", 3], ["aa", 5], ["bb", 6], ["cc", 7]], HashMap_1.HashMap.of(["a", 1], ["b", 2], ["c", 3])
        .flatMap(function (k, v) { return HashMap_1.HashMap.of([k + k, v + 4], [k, v]); }).toVector().sortOn(function (x) { return x[1]; }).toArray()); });
    it("should transform through mapValues", function () { return assert.ok(HashMap_1.HashMap.empty().put("key1", 12).put("key2", 6).equals(HashMap_1.HashMap.empty().put("key1", 6).put("key2", 3).mapValues(function (v) { return v * 2; }))); });
    it("should transform through empty mapValues", function () { return assert.ok(HashMap_1.HashMap.empty().equals(HashMap_1.HashMap.empty().mapValues(function (v) { return v * 2; }))); });
    it("should transform non-empty to vector", function () { return assert.deepEqual([["a", 1], ["b", 2]], HashMap_1.HashMap.empty().put("a", 1).put("b", 2).toVector().toArray()); });
    it("should transform empty to vector", function () { return assert.deepEqual([], HashMap_1.HashMap.empty().toVector().toArray()); });
    it("should transform non-empty to array", function () { return assert.deepEqual([["a", 1], ["b", 2]], HashMap_1.HashMap.empty().put("a", 1).put("b", 2).toArray()); });
    it("should transform empty to array", function () { return assert.deepEqual([], HashMap_1.HashMap.empty().toArray()); });
    it("should transform non-empty to ObjectDictionary", function () { return assert.deepEqual({ "a": 1, "b": 2 }, HashMap_1.HashMap.empty().put("a", 1).put("b", 2).toObjectDictionary(function (x) { return x; })); });
    it("should transform empty to ObjectDictionary", function () { return assert.deepEqual({}, HashMap_1.HashMap.empty().toObjectDictionary(function (x) { return x; })); });
    it("should filter properly", function () { return assert.deepEqual([[1, "a"], [3, "c"]], HashMap_1.HashMap.empty()
        .put(1, "a").put(2, "b").put(3, "c").put(4, "d").filter(function (k, v) { return k % 2 != 0; }).toArray()); });
    it("should filter empty properly", function () { return assert.deepEqual([], HashMap_1.HashMap.empty().toArray()); });
    it("keeps the custom equality even after filter", function () { return assert.ok(HashMap_1.HashMap.empty()
        .put(new SampleData_1.MyClass("a", 1), "test")
        .put(new SampleData_1.MyClass("a", 2), "test1").equals(HashMap_1.HashMap.of([new SampleData_1.MyClass("a", 1), "test1"])
        .filter(function (x) { return true; })
        .put(new SampleData_1.MyClass("a", 1), "test")
        .put(new SampleData_1.MyClass("a", 2), "test1"))); });
    it("should support allMatch, positive case", function () { return assert.ok(HashMap_1.HashMap.empty().put(1, "a").put(2, "b").allMatch(function (k, v) { return k > 0; })); });
    it("should support allMatch, negative case", function () { return assert.ok(!HashMap_1.HashMap.empty().put(1, "a").put(2, "b").allMatch(function (k, v) { return k < 0; })); });
    it("should support allMatch, empty map", function () { return assert.ok(HashMap_1.HashMap.empty().allMatch(function (k, v) { return k > 0; })); });
    it("should support anyMatch, positive case", function () { return assert.ok(HashMap_1.HashMap.empty().put(1, "a").put(-1, "b").anyMatch(function (k, v) { return k > 0; })); });
    it("should support anyMatch, negative case", function () { return assert.ok(!HashMap_1.HashMap.empty().put(1, "a").put(2, "b").anyMatch(function (k, v) { return k < 0; })); });
    it("should support anyMatch, empty map", function () { return assert.ok(!HashMap_1.HashMap.empty().anyMatch(function (k, v) { return k > 0; })); });
    it("should support contains, positive case", function () { return assert.ok(HashMap_1.HashMap.empty().put(1, "a").put(2, "b").contains([2, "b"])); });
    it("should support contains, negative case", function () { return assert.ok(!HashMap_1.HashMap.empty().put(1, "a").put(2, "b").contains([2, "c"])); });
    it("should support contains, empty map", function () { return assert.ok(!HashMap_1.HashMap.empty().contains([2, "b"])); });
    it("should fold correctly", function () { return assert.deepEqual([6, "c"], HashMap_1.HashMap.of([1, "a"], [2, "b"], [3, "c"])
        .fold([0, ""], function (_a, _b) {
        var a = _a[0], b = _a[1];
        var c = _b[0], d = _b[1];
        return [a + c, b > d ? b : d];
    })); });
    it("should foldLeft correctly", function () { return assert.equal(6, HashMap_1.HashMap.of([1, "a"], [2, "bb"], [3, "ccc"])
        .foldLeft(0, function (soFar, _a) {
        var item = _a[0], val = _a[1];
        return soFar + val.length;
    })); });
    it("should foldRight correctly", function () { return assert.equal(6, HashMap_1.HashMap.of([1, "a"], [2, "bb"], [3, "ccc"])
        .foldRight(0, function (_a, soFar) {
        var item = _a[0], value = _a[1];
        return soFar + value.length;
    })); });
    it("should convert to linked list correctly", function () { return assert.ok(LinkedList_1.LinkedList.of(Tuple2_1.Tuple2.of("a", 5), Tuple2_1.Tuple2.of("b", 6)).equals(HashMap_1.HashMap.of(["a", 5], ["b", 6]).toLinkedList().map(Tuple2_1.Tuple2.ofPair))); });
    it("reduce works", function () { return assert.deepEqual([3, "c"], HashMap_1.HashMap.of([1, "a"], [2, "b"], [3, "c"])
        .reduce(function (kv1, kv2) { return kv1[0] > kv2[0] ? kv1 : kv2; }).getOrThrow()); });
    it("reduce works on an empty collection", function () { return assert.ok(HashMap_1.HashMap.empty().reduce(function (kv1, kv2) { return kv1[0] > kv2[0] ? kv1 : kv2; }).isNone()); });
});
//# sourceMappingURL=HashMap.js.map