"use strict";
exports.__esModule = true;
var Lazy_1 = require("../src/Lazy");
var assert = require("assert");
describe("lazy basics", function () {
    it("should be lazy", function () {
        var evaluated = false;
        var val = function () {
            evaluated = true;
            return 2;
        };
        var l = Lazy_1.Lazy.of(val);
        assert.equal(false, evaluated);
        assert.equal(2, l.get());
        assert.equal(true, evaluated);
        // should get the same when it's cashed
        // but shouldn't evaluate again
        evaluated = false;
        assert.equal(2, l.get());
        assert.equal(false, evaluated);
    });
    it("map should be lazy", function () {
        var evaluated = false;
        var val = function () {
            evaluated = true;
            return 2;
        };
        var l = Lazy_1.Lazy.of(val);
        var l2 = l.map(function (x) { return x * 2; });
        assert.equal(false, evaluated);
        assert.equal(4, l2.get());
        assert.equal(true, evaluated);
    });
    it("should convert to string properly, value not present", function () {
        return assert.equal("Lazy(?)", Lazy_1.Lazy.of(function () { return 5; }).toString());
    });
    it("should convert to string properly, value present", function () {
        var l = Lazy_1.Lazy.of(function () { return 5; });
        l.get();
        assert.equal("Lazy(5)", l.toString());
    });
});
//# sourceMappingURL=Lazy.js.map