"use strict";
exports.__esModule = true;
var Stream_1 = require("../src/Stream");
var SampleData_1 = require("./SampleData");
var SeqTest = require("./Seq");
var assert = require("assert");
SeqTest.runTests("Stream", Stream_1.Stream.ofIterable, Stream_1.Stream.of, Stream_1.Stream.empty, Stream_1.Stream.unfoldRight, "ConsStream");
describe("Stream basics", function () {
    it("creates a continually constant value", function () { return assert.deepEqual([1, 1, 1, 1], Stream_1.Stream.continually(function () { return 1; }).take(4).toArray()); });
    it("iterates from a seed", function () { return assert.deepEqual([1, 2, 4, 8], Stream_1.Stream.iterate(1, function (x) { return x * 2; }).take(4).toArray()); });
    it("maps lazily correctly", function () { return assert.deepEqual([4, 5, 7, 11], Stream_1.Stream.iterate(1, function (x) { return x * 2; }).map(function (x) { return x + 3; }).take(4).toArray()); });
    it("supports appendStream", function () { return assert.deepEqual([1, 2, 3, 4, 5, 6], Stream_1.Stream.of(1, 2, 3).appendStream(Stream_1.Stream.of(4, 5, 6)).toArray()); });
    it("supports cycle", function () { return assert.deepEqual([1, 2, 3, 1, 2, 3, 1, 2], Stream_1.Stream.of(1, 2, 3).cycle().take(8).toArray()); });
    it("takes advantage of isEmpty", function () {
        var stream = Stream_1.Stream.of(1, 2, 3);
        if (!stream.isEmpty()) {
            stream.head().get();
        }
    });
});
describe("Stream filtering", function () {
    it("implements takeWhile correctly", function () { return assert.deepEqual([1, 2, 3], Stream_1.Stream.iterate(1, function (x) { return x + 1; }).takeWhile(function (x) { return x < 4; }).toArray()); });
    it("filters lazily correctly", function () { return assert.deepEqual([8, 32, 64, 128], Stream_1.Stream.iterate(1, function (x) { return x * 2; }).filter(function (x) { return x > 5 && (x < 15 || x > 30); }).take(4).toArray()); });
});
describe("Stream toString", function () {
    it("implements toString correctly on infinite streams", function () { return assert.equal("Stream(1, 2, ?)", (function () {
        var s = Stream_1.Stream.iterate(1, function (x) { return x + 1; });
        s.get(1);
        return s.toString();
    })()); });
    it("implements toString ok on fully-evaluated", function () { return assert.equal("Stream(1, 2, 3)", (function () {
        var s = Stream_1.Stream.iterate(1, function (x) { return x + 1; }).take(3);
        s.length();
        return s.toString();
    })()); });
    it("implements toString ok on fully-lazy", function () { return assert.equal("Stream(1, ?)", Stream_1.Stream.iterate(1, function (x) { return x + 1; }).take(3)); });
    it("serializes to string correctly", function () { return assert.equal("Stream(1, ?)", Stream_1.Stream.of(1, 2, 3).toString()); });
    it("serializes to string correctly - arrays & strings", function () { return assert.equal("Stream([1,'a'], ?)", Stream_1.Stream.of([1, 'a']).toString()); });
    it("serializes to string correctly - custom toString", function () { return assert.equal("Stream({field1: hi, field2: 99}, ?)", Stream_1.Stream.of(new SampleData_1.MyClass("hi", 99)).toString()); });
    it("serializes to string correctly - plain map", function () { return assert.equal("Stream({\"name\":\"hi\",\"age\":99}, ?)", Stream_1.Stream.of({ name: "hi", age: 99 }).toString()); });
});
//# sourceMappingURL=Stream.js.map