"use strict";
exports.__esModule = true;
var ts = require("typescript");
var fs = require('fs');
var assert = require("assert");
var TMP_FILENAME = "tmp.ts";
function diagnosticMsgToString(diagMsg) {
    if (typeof diagMsg === "string") {
        return diagMsg;
    }
    var curMsg = diagMsg;
    while (curMsg) {
        if (typeof curMsg.messageText === "string") {
            return curMsg.messageText;
        }
        curMsg = curMsg.next;
    }
    return "found no error";
}
function diagnosticMsgContains(diagMsg, contents) {
    return diagnosticMsgToString(diagMsg).indexOf(contents) >= 0;
}
/**
 * @hidden
 */
function assertFailCompile(contents, expectedMsg) {
    fs.writeFileSync(TMP_FILENAME, "import { HashSet } from './dist/src/HashSet';" +
        " import { Stream } from './dist/src/Stream';" +
        " import { LinkedList } from './dist/src/LinkedList';" +
        " import { HashMap } from './dist/src/HashMap';" +
        " import { Option } from './dist/src/Option';" +
        " import { Either } from './dist/src/Either';" +
        " import { Vector } from './dist/src/Vector';" + contents);
    var tsProgram = ts.createProgram([TMP_FILENAME], { target: ts.ScriptTarget.ES2016 });
    var emitResult = tsProgram.emit();
    var allDiagnostics = ts.getPreEmitDiagnostics(tsProgram)
        .concat(emitResult.diagnostics);
    var allErrorsTxt = allDiagnostics.map(function (x) { return diagnosticMsgToString(x.messageText); }).join(", ");
    // for some reason getting tons of 'cannot find module' errors
    // on circleCI. The test is otherwise still valid since i do "contains"
    // and the real error is still in there.
    //
    // if (allDiagnostics.length > 1) {
    //     console.log(allErrorsTxt);
    // }
    // assert.equal(1, allDiagnostics.length);
    var isMatch = allDiagnostics.filter(function (d) { return diagnosticMsgContains(d.messageText, expectedMsg); }).length > 0;
    if (isMatch) {
        assert.ok(true);
    }
    else {
        assert.equal(allErrorsTxt, expectedMsg);
    }
    fs.unlinkSync(TMP_FILENAME);
    fs.unlinkSync(TMP_FILENAME.replace(/.ts$/, ".js"));
}
exports.assertFailCompile = assertFailCompile;
//# sourceMappingURL=TestHelpers.js.map